%global upstreamver 2-5-4

Name:           cxsc
Version:        %(tr - . <<< %{upstreamver})
Release:        1%{?dist}
Summary:        C++ library for Extended Scientific Computing

%global majver  %(cut -d. -f1 <<< %{version})

License:        LGPLv2+
URL:            http://www2.math.uni-wuppertal.de/wrswt/xsc/cxsc_new.html
Source0:        http://www2.math.uni-wuppertal.de/wrswt/xsc/%{name}/%{name}-%{upstreamver}.tar.gz
# Sent upstream 22 Jun 2016.  Fix an operator error.
Patch0:         %{name}-operator.patch
# Sent upstream 22 Jun 2016.  Fix build problem on ppc64.
Patch1:         %{name}-ppc64.patch

BuildRequires:  atlas-devel
BuildRequires:  doxygen-latex
BuildRequires:  gcc-c++
BuildRequires:  ghostscript-core

%description
C-XSC is the C language variant of the XSC (eXtensions for Scientific
Computing) project.  It provides routines that guarantee accuracy and
reliability of results.  Problem-solving routines with automatic result
verification have been developed for many standard problems of numerical
analysis, such as linear or nonlinear systems of equations, differential
and integral equations, etc. as well as for a large number of
applications in engineering and the natural sciences.  Some of the
features of C-XSC are:
- Operator concept (user-defined operators)
- Overloading concept
- Module concept
- Dynamic arrays
- Controlled rounding
- Predefined arithmetic data types real, extended real, complex,
  interval, complex interval, and corresponding vector and matrix types
- Predefined arithmetic operators and elementary functions of the highest
  accuracy for the arithmetic data types
- Data type dotprecision for the exact representation of dot products
- Library of mathematical problem-solving routines with automatic result
  verification and high accuracy

%package devel
Summary:        Header files for developing applications that use %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description devel
Header files and library links for developing applications that use %{name}.

%package doc
Summary:        API documentation for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}
Provides:       bundled(jquery)

%description doc
API documentation for %{name}.

%prep
%setup -q -n %{name}-%{upstreamver}
%patch0
%patch1

# Don't set rpath
sed -i 's/\(RPATH[[:blank:]]*=\).*/\1/;' Makefile.in CToolbox/Makefile
sed -i '/LINKERPATH=-Wl,-R/d' install_cxsc

# Don't build with SSE2 support on platforms the script doesn't recognize
%ifnarch %{ix86} x86_64
sed -i 's/ -mfpmath=sse -msse2//' install_cxsc.in
%endif

# Link with the atlas and OpenMP libraries
sed -i 's/\$(RARI)/& -lsatlas -lgomp/' src/Makefile
sed -i 's/(LIBS)/& -lsatlas/' CToolbox/Makefile

# Install in the right place on 64-bit systems
if [ %{_libdir} != "%{_prefix}/lib" ]; then
  sed -e 's|\$(PREFIX)/lib$|$(PREFIX)/%{_lib}|' \
      -e 's|\$(PREFIX)/lib;|$(PREFIX)/%{_lib};|' \
      -i src/Makefile
fi

# Remove spurious executable bits
chmod a-x src/fi_lib/*.{cpp,hpp}

# Prepare to build both SSE2 and non-SSE2 variants on 32-bit x86
%ifarch %{ix86}
cp -a src src2
%endif

%build
# FIXME: tests fail without -fno-inline.  Why?
printf "yes\n\
gnu\n\
no\n\
yes\n\
%ifarch x86_64
%{optflags} -DCXSC_USE_BLAS -DCXSC_USE_LAPACK -DCXSC_USE_OPENMP -DCXSC_USE_FMA -fopenmp -L%{_libdir}/atlas -Wl,--as-needed\n\
64\n\
asm\n\
%else
%ifarch %{ix86} ppc64 ppc64le
%{optflags} -DCXSC_USE_BLAS -DCXSC_USE_LAPACK -DCXSC_USE_OPENMP -DCXSC_USE_FMA -fopenmp -L%{_libdir}/atlas -Wl,--as-needed\n\
asm\n\
%else
%{optflags} -DCXSC_USE_BLAS -DCXSC_USE_LAPACK -DCXSC_USE_OPENMP -DCXSC_USE_FMA -fopenmp -frounding-math -fno-inline -L%{_libdir}/atlas -Wl,--as-needed\n\
hard\n\
safe\n\
%endif
%endif
%{buildroot}%{_prefix}\n\
dynamic\n\
no\n" | ./install_cxsc

# The individual targets can be built in parallel, but specifying more than one
# to the same make invocation leads to build failures.
make %{?_smp_mflags} cxsc
make %{?_smp_mflags} libcxsc.so
mkdir usr
ln -s ../src usr/lib
ln -s lib%{name}.so.%{version} src/lib%{name}.so.%{majver}
ln -s lib%{name}.so.%{majver} src/lib%{name}.so
export LD_LIBRARY_PATH=$PWD/usr/lib
make %{?_smp_mflags} toolbox_dyn CXSCDIR=$PWD/usr

# Build a non-SSE2 variant for 32-bit x86
%ifarch %{ix86}
unset LD_LIBRARY_PATH
mv src src.sse2
mv src2 src
sed -i 's/ -mfpmath=sse -msse2//' install_cxsc.in
printf "yes\n\
gnu\n\
no\n\
yes\n\
%{optflags} -DCXSC_USE_BLAS -DCXSC_USE_LAPACK -DCXSC_USE_OPENMP -DCXSC_USE_FMA -fopenmp -frounding-math -fno-inline -L%{_libdir}/atlas -Wl,--as-needed\n\
hard\n\
safe\n\
%{buildroot}%{_prefix}\n\
dynamic\n\
no\n" | ./install_cxsc
make %{?_smp_mflags} cxsc
make %{?_smp_mflags} libcxsc.so
ln -s lib%{name}.so.%{version} src/lib%{name}.so.%{majver}
ln -s lib%{name}.so.%{majver} src/lib%{name}.so
export LD_LIBRARY_PATH=$PWD/usr/lib
make %{?_smp_mflags} toolbox_dyn CXSCDIR=$PWD/usr
%endif

# Make the documentation
cd src
doxygen src-doxyfile

%install
make install_dyn PREFIX=%{buildroot}%{_prefix}

# Fix permissions on the library
chmod 0755 %{buildroot}%{_libdir}/lib%{name}.so.%{version}

# Install the SSE2 variant on 32-bit x86
%ifarch %{ix86}
mkdir -p %{buildroot}%{_libdir}/sse2
install -m 0755 -p src.sse2/lib%{name}.so.%{version} %{buildroot}%{_libdir}/sse2
ln -s lib%{name}.so.%{version} \
   %{buildroot}%{_libdir}/sse2/lib%{name}.so.%{majver}
%endif

# There are a lot of header files, so hide them in a private directory
mkdir %{buildroot}%{_includedir}/%{name}
mv %{buildroot}%{_includedir}/*.{h,hpp,inl} %{buildroot}%{_includedir}/%{name}

# Don't package the example binaries
rm -fr %{buildroot}/%{_prefix}/examples

%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig

%check
sed -i 's/ASM$/ASM LD_LIBRARY_PATH/' Makefile
sed -i 's/export RPATH/export LD_LIBRARY_PATH/' CToolbox/Makefile
if [ %{_libdir} != "%{_prefix}/lib" ]; then
  sed -i 's|/lib|/%{_lib}|' CToolbox/Makefile
fi
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}
make toolboxtest_dyn

%files
%doc changelog README
%license docu/COPYING
%{_libdir}/lib%{name}.so.2*
%ifarch %{ix86}
%{_libdir}/sse2/lib%{name}.so.2*
%endif

%files devel
%doc examples
%{_includedir}/%{name}/
%{_libdir}/lib%{name}.so

%files doc
%doc docu/apidoc docu/images

%changelog
* Wed Jun 22 2016 Jerry James <loganjerry@gmail.com> - 2.5.4-1
- Initial RPM
